@@ -61,7 +61,6 @@ class CASCache:
         os.makedirs(self.tmpdir, exist_ok=True)
 
         self._cache_usage_monitor = None
-        self._cache_usage_monitor_forbidden = False
 
         self._remote_cache = remote_cache
 
@@ -125,16 +124,15 @@ class CASCache:
 
     # contains_directory():
     #
-    # Check whether the specified directory and subdirectories are in the cache,
-    # i.e non dangling.
+    # Check whether the specified directory, subdirectories and files are in the
+    # cache, i.e non dangling.
     #
     # Args:
     #     digest (Digest): The directory digest to check
-    #     with_files (bool): Whether to check files as well
     #
     # Returns: True if the directory is available in the local cache
     #
-    def contains_directory(self, digest, *, with_files):
+    def contains_directory(self, digest):
         local_cas = self.get_local_cas()
 
         # Without a remote cache, `FetchTree` simply checks the local cache.
@@ -142,7 +140,7 @@ class CASCache:
         request.root_digest.CopyFrom(digest)
         # Always fetch Directory protos as they are needed to enumerate subdirectories and files.
         # Don't implicitly fetch file blobs from the remote cache as we don't need them.
-        request.fetch_file_blobs = with_files and not self._remote_cache
+        request.fetch_file_blobs = not self._remote_cache
 
         try:
             local_cas.FetchTree(request)
@@ -155,9 +153,20 @@ class CASCache:
                 raise CASCacheError("Unsupported buildbox-casd version: FetchTree unimplemented") from e
             raise
 
-        # Check whether everything is available in the remote cache.
-        missing_blobs = self.missing_blobs_for_directory(digest, remote=self._default_remote)
-        return not missing_blobs
+        # Make sure everything is available in the remote cache (storage-service)
+        request = local_cas_pb2.UploadTreeRequest()
+        request.root_digest.CopyFrom(digest)
+        try:
+            local_cas.UploadTree(request)
+            return True
+        except grpc.RpcError as e:
+            if e.code() == grpc.StatusCode.NOT_FOUND:
+                return False
+            if e.code() == grpc.StatusCode.UNIMPLEMENTED:
+                # Fallback path if buildbox-casd is too old to support UploadTree
+                missing_blobs = self.missing_blobs_for_directory(digest, remote=self._default_remote)
+                return not missing_blobs
+            raise
 
     # checkout():
     #
@@ -217,6 +226,11 @@ class CASCache:
             fullpath = os.path.join(dest, symlinknode.name)
             os.symlink(symlinknode.target, fullpath)
 
+        node_properties = directory.node_properties
+        if node_properties.HasField("mtime"):
+            mtime = utils._parse_protobuf_timestamp(node_properties.mtime)
+            utils._set_file_mtime(dest, mtime)
+
     # ensure_tree():
     #
     # Make sure all blobs referenced by the given directory tree are available
@@ -235,6 +249,39 @@ class CASCache:
 
             local_cas.FetchTree(request)
 
+    # fetch_directory():
+    #
+    # Fetches remote directory and adds it to content addressable store.
+    #
+    # This recursively fetches directory objects and files.
+    #
+    # Args:
+    #     remote (Remote): The remote to use.
+    #     dir_digest (Digest): Digest object for the directory to fetch.
+    #
+    def fetch_directory(self, remote, dir_digest):
+        local_cas = self.get_local_cas()
+
+        request = local_cas_pb2.FetchTreeRequest()
+        request.instance_name = remote.local_cas_instance_name
+        request.root_digest.CopyFrom(dir_digest)
+        request.fetch_file_blobs = False
+
+        try:
+            local_cas.FetchTree(request)
+        except grpc.RpcError as e:
+            if e.code() == grpc.StatusCode.NOT_FOUND:
+                raise BlobNotFound(
+                    dir_digest.hash,
+                    "Failed to fetch directory tree {}: {}: {}".format(dir_digest.hash, e.code().name, e.details()),
+                ) from e
+            raise CASCacheError(
+                "Failed to fetch directory tree {}: {}: {}".format(dir_digest.hash, e.code().name, e.details())
+            ) from e
+
+        required_blobs = self.required_blobs_for_directory(dir_digest)
+        self.fetch_blobs(remote, required_blobs)
+
     # pull_tree():
     #
     # Pull a single Tree rather than a ref.
@@ -569,39 +616,6 @@ class CASCache:
             os.chmod(f.name, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH)
             yield f
 
-    # _fetch_directory():
-    #
-    # Fetches remote directory and adds it to content addressable store.
-    #
-    # This recursively fetches directory objects and files.
-    #
-    # Args:
-    #     remote (Remote): The remote to use.
-    #     dir_digest (Digest): Digest object for the directory to fetch.
-    #
-    def _fetch_directory(self, remote, dir_digest):
-        local_cas = self.get_local_cas()
-
-        request = local_cas_pb2.FetchTreeRequest()
-        request.instance_name = remote.local_cas_instance_name
-        request.root_digest.CopyFrom(dir_digest)
-        request.fetch_file_blobs = False
-
-        try:
-            local_cas.FetchTree(request)
-        except grpc.RpcError as e:
-            if e.code() == grpc.StatusCode.NOT_FOUND:
-                raise BlobNotFound(
-                    dir_digest.hash,
-                    "Failed to fetch directory tree {}: {}: {}".format(dir_digest.hash, e.code().name, e.details()),
-                ) from e
-            raise CASCacheError(
-                "Failed to fetch directory tree {}: {}: {}".format(dir_digest.hash, e.code().name, e.details())
-            ) from e
-
-        required_blobs = self.required_blobs_for_directory(dir_digest)
-        self.fetch_blobs(remote, required_blobs)
-
     def _fetch_tree(self, remote, digest):
         self.fetch_blobs(remote, [digest])
 
@@ -688,7 +702,7 @@ class CASCache:
         batch.send()
 
     def _send_directory(self, remote, digest):
-        required_blobs = self.required_blobs_for_directory(digest)
+        required_blobs = list(self.required_blobs_for_directory(digest))
 
         # Upload any blobs missing on the server.
         # buildbox-casd will call FindMissingBlobs before the actual upload
@@ -703,7 +717,6 @@ class CASCache:
     #     (CASCacheUsage): The current status
     #
     def get_cache_usage(self):
-        assert not self._cache_usage_monitor_forbidden
         return self._cache_usage_monitor.get_cache_usage()
 
     # get_casd()
