@@ -20,6 +20,7 @@ import random
 import shutil
 import stat
 import subprocess
+import sys
 import tempfile
 import time
 from subprocess import CalledProcessError
@@ -44,9 +45,9 @@ _CASD_TIMEOUT = 300  # in seconds
 #
 # Minimum required version of buildbox-casd
 #
-_REQUIRED_CASD_MAJOR = 0
-_REQUIRED_CASD_MINOR = 0
-_REQUIRED_CASD_MICRO = 58
+_REQUIRED_CASD_MAJOR = 1
+_REQUIRED_CASD_MINOR = 2
+_REQUIRED_CASD_MICRO = 6
 
 
 # CASDProcessManager
@@ -58,12 +59,26 @@ _REQUIRED_CASD_MICRO = 58
 #     log_dir (str): The directory for the logs
 #     log_level (LogLevel): Log level to give to buildbox-casd for logging
 #     cache_quota (int): User configured cache quota
+#     reserved (int): User configured reserved disk space
 #     remote_cache_spec (RemoteSpec): Optional remote cache server
 #     protect_session_blobs (bool): Disable expiry for blobs used in the current session
 #     messenger (Messenger): The messenger to report warnings through the UI
 #
 class CASDProcessManager:
-    def __init__(self, path, log_dir, log_level, cache_quota, remote_cache_spec, protect_session_blobs, messenger):
+    def __init__(
+        self,
+        path,
+        log_dir,
+        log_level,
+        cache_quota,
+        remote_cache_spec,
+        protect_session_blobs,
+        messenger,
+        *,
+        reserved=None,
+        low_watermark=None,
+        local_jobs=None
+    ):
         os.makedirs(path, exist_ok=True)
 
         self._log_dir = log_dir
@@ -80,11 +95,27 @@ class CASDProcessManager:
 
         if cache_quota is not None:
             casd_args.append("--quota-high={}".format(int(cache_quota)))
-            casd_args.append("--quota-low={}".format(int(cache_quota / 2)))
+
+        if low_watermark is not None:
+            casd_args.append("--quota-low={}%".format(int(low_watermark * 100)))
+
+        if reserved is not None:
+            casd_args.append("--reserved={}".format(int(reserved)))
 
         if protect_session_blobs:
             casd_args.append("--protect-session-blobs")
 
+        if local_jobs is not None:
+            try:
+                buildbox_run = utils._get_host_tool_internal("buildbox-run", search_subprojects_dir="buildbox")
+                casd_args.append("--buildbox-run={}".format(buildbox_run))
+                casd_args.append("--jobs={}".format(local_jobs))
+            except utils.ProgramNotFoundError:
+                # Not fatal as buildbox-run is not needed for remote execution
+                # and buildbox-casd local execution will never be used if
+                # buildbox-run is not available.
+                pass
+
         if remote_cache_spec:
             casd_args.append("--cas-remote={}".format(remote_cache_spec.url))
             if remote_cache_spec.instance_name:
@@ -102,22 +133,33 @@ class CASDProcessManager:
                     )
             if remote_cache_spec.keepalive_time is not None:
                 casd_args.append("--cas-keepalive-time={}".format(remote_cache_spec.keepalive_time))
+            if remote_cache_spec.retry_limit is not None:
+                casd_args.append("--cas-retry-limit={}".format(remote_cache_spec.retry_limit))
+            if remote_cache_spec.retry_delay is not None:
+                casd_args.append("--cas-retry-delay={}".format(remote_cache_spec.retry_delay))
+            if remote_cache_spec.request_timeout is not None:
+                casd_args.append("--cas-request-timeout={}".format(remote_cache_spec.request_timeout))
 
         casd_args.append(path)
 
         self._start_time = time.time()
         self._logfile = self._rotate_and_get_next_logfile()
 
+        # Create a new process group for buildbox-casd such that SIGINT won't reach it.
+        if sys.version_info >= (3, 11):
+            process_group_kwargs = {"process_group": 0}
+        else:
+            process_group_kwargs = {"preexec_fn": os.setpgrp}
+
         with open(self._logfile, "w", encoding="utf-8") as logfile_fp:
             # The frontend will take care of terminating buildbox-casd.
-            # Create a new process group for it such that SIGINT won't reach it.
-            self.process = subprocess.Popen(  # pylint: disable=consider-using-with, subprocess-popen-preexec-fn
+            self.process = subprocess.Popen(  # pylint: disable=consider-using-with
                 casd_args,
                 cwd=path,
                 stdout=logfile_fp,
                 stderr=subprocess.STDOUT,
-                preexec_fn=os.setpgrp,
                 env=self.__buildbox_casd_env(),
+                **process_group_kwargs
             )
 
         self._casd_channel = None
