@@ -185,6 +185,12 @@ class Context:
         # User specified cache quota, used for display messages
         self.config_cache_quota_string: Optional[str] = None
 
+        # Reserved disk space for local cache in bytes
+        self.config_cache_reserved: Optional[int] = None
+
+        # Low watermark for local cache (ratio relative to effective quota)
+        self.config_cache_low_watermark: Optional[float] = None
+
         # Remote cache server
         self.remote_cache_spec: Optional[RemoteSpec] = None
 
@@ -238,15 +244,6 @@ class Context:
     # Called when exiting the with-statement context.
     #
     def __exit__(self, exc_type, exc_value, traceback) -> None:
-        if self._artifactcache:
-            self._artifactcache.release_resources()
-
-        if self._elementsourcescache:
-            self._elementsourcescache.release_resources()
-
-        if self._sourcecache:
-            self._sourcecache.release_resources()
-
         if self._cascache:
             self._cascache.release_resources()
 
@@ -362,7 +359,9 @@ class Context:
         # We need to find the first existing directory in the path of our
         # casdir - the casdir may not have been created yet.
         cache = defaults.get_mapping("cache")
-        cache.validate_keys(["quota", "storage-service", "pull-buildtrees", "cache-buildtrees"])
+        cache.validate_keys(
+            ["quota", "reserved-disk-space", "low-watermark", "storage-service", "pull-buildtrees", "cache-buildtrees"]
+        )
 
         cas_volume = self.casdir
         while not os.path.exists(cas_volume):
@@ -378,6 +377,32 @@ class Context:
                 LoadErrorReason.INVALID_DATA,
             ) from e
 
+        cache_reserved_string = cache.get_str("reserved-disk-space")
+        try:
+            self.config_cache_reserved = utils._parse_size(cache_reserved_string, cas_volume)
+            if self.config_cache_reserved is None:
+                provenance = cache.get_scalar("reserved-disk-space").get_provenance()
+                raise LoadError(
+                    "{}: Please specify the value in bytes or as a % of full disk space.\n"
+                    "\nValid values are, for example: 2G 5%\n".format(provenance),
+                    LoadErrorReason.INVALID_DATA,
+                )
+        except utils.UtilError as e:
+            raise LoadError(
+                "{}\nPlease specify the value in bytes or as a % of full disk space.\n"
+                "\nValid values are, for example: 2G 5%\n".format(str(e)),
+                LoadErrorReason.INVALID_DATA,
+            ) from e
+
+        low_watermark_string = cache.get_str("low-watermark")
+        try:
+            self.config_cache_low_watermark = utils._parse_percentage(low_watermark_string)
+        except utils.UtilError as e:
+            raise LoadError(
+                "{}\nPlease specify the value as a % of the cache quota.".format(str(e)),
+                LoadErrorReason.INVALID_DATA,
+            ) from e
+
         remote_cache = cache.get_mapping("storage-service", default=None)
         if remote_cache:
             self.remote_cache_spec = RemoteSpec.new_from_node(remote_cache)
@@ -497,6 +522,13 @@ class Context:
 
         return self._sourcecache
 
+    @property
+    def effective_build_max_jobs(self) -> int:
+        # Based on some testing (mainly on AWS), maximum effective
+        # max-jobs value seems to be around 8-10 if we have enough cores
+        # users should set values based on workload and build infrastructure
+        return self.build_max_jobs or self.platform.get_cpu_count(8)
+
     # add_project():
     #
     # Add a project to the context.
@@ -693,6 +725,7 @@ class Context:
 
             assert self.logdir is not None, "log_directory is required for casd"
             log_dir = os.path.join(self.logdir, "_casd")
+            assert self.sched_builders is not None, "builders configuration is required"
             self._casd = CASDProcessManager(
                 self.cachedir,
                 log_dir,
@@ -701,6 +734,9 @@ class Context:
                 self.remote_cache_spec,
                 protect_session_blobs=True,
                 messenger=self.messenger,
+                reserved=self.config_cache_reserved,
+                low_watermark=self.config_cache_low_watermark,
+                local_jobs=self.sched_builders * self.effective_build_max_jobs,
             )
         return self._casd
 
