@@ -15,6 +15,7 @@ import os
 import re
 import sys
 from functools import partial
+from typing import TYPE_CHECKING
 
 import shutil
 import click
@@ -25,6 +26,15 @@ from ..types import _CacheBuildTrees, _SchedulerErrorAction, _PipelineSelection,
 from .._remotespec import RemoteSpec, RemoteSpecPurpose
 from ..utils import UtilError
 
+if TYPE_CHECKING or click.Command.__bases__ == (object,):
+    # Click >= 8.2
+    ClickCommandBaseClass = click.Command
+    ClickGroupBaseClass = click.Group
+else:
+    # Click < 8.2
+    ClickCommandBaseClass = click.BaseCommand
+    ClickGroupBaseClass = click.MultiCommand
+
 
 ##################################################################
 #              Helper classes and methods for Click              #
@@ -105,7 +115,7 @@ def search_command(args, *, context=None):
 # Completion for completing command names as help arguments
 def complete_commands(cmd, args, incomplete):
     command_ctx = search_command(args[1:])
-    if command_ctx and command_ctx.command and isinstance(command_ctx.command, click.MultiCommand):
+    if command_ctx and command_ctx.command and isinstance(command_ctx.command, ClickGroupBaseClass):
         return [
             subcommand + " "
             for subcommand in command_ctx.command.list_commands(command_ctx)
@@ -272,10 +282,10 @@ def override_main(self, args=None, prog_name=None, complete_var=None, standalone
     original_main(self, args=args, prog_name=prog_name, complete_var=None, standalone_mode=standalone_mode, **extra)
 
 
-original_main = click.BaseCommand.main
+original_main = ClickCommandBaseClass.main
 # Disable type checking since mypy doesn't support assigning to a method.
 # See https://github.com/python/mypy/issues/2427.
-click.BaseCommand.main = override_main  # type: ignore
+ClickCommandBaseClass.main = override_main  # type: ignore
 
 
 ##################################################################
@@ -392,7 +402,7 @@ def help_command(ctx, command):
     click.echo(command_ctx.command.get_help(command_ctx), err=True)
 
     # Hint about available sub commands
-    if isinstance(command_ctx.command, click.MultiCommand):
+    if isinstance(command_ctx.command, ClickGroupBaseClass):
         detail = " "
         if command:
             detail = " {} ".format(" ".join(command))
@@ -409,7 +419,7 @@ def help_command(ctx, command):
 @click.option(
     "--min-version",
     type=click.STRING,
-    default="2.3",
+    default="2.6",
     show_default=True,
     help="The required format version",
 )
@@ -600,6 +610,7 @@ def show(app, elements, deps, except_, order, format_):
 
     \b
         %{name}           The element name
+        %{kind}           The element kind (Since: 2.6)
         %{description}    The element description, on a single line (Since: 2.3)
         %{key}            The abbreviated cache key (if all sources are consistent)
         %{full-key}       The full cache key (if all sources are consistent)
@@ -613,6 +624,8 @@ def show(app, elements, deps, except_, order, format_):
         %{deps}           A list of all dependencies
         %{build-deps}     A list of build dependencies
         %{runtime-deps}   A list of runtime dependencies
+        %{source-info}    Source provenance information
+        %{artifact-cas-digest} The CAS digest of the built artifact
 
     The value of the %{symbol} without the leading '%' character is understood
     as a pythonic formatting string, so python formatting features apply,
@@ -638,7 +651,8 @@ def show(app, elements, deps, except_, order, format_):
         state_match = re.search(r"%(\{(state)[^%]*?\})", format_)
         key_match = re.search(r"%(\{(key)[^%]*?\})", format_)
         full_key_match = re.search(r"%(\{(full-key)[^%]*?\})", format_)
-        need_state = bool(state_match or key_match or full_key_match)
+        artifact_cas_digest_match = re.search(r"%(\{(artifact-cas-digest)[^%]*?\})", format_)
+        need_state = bool(state_match or key_match or full_key_match or artifact_cas_digest_match)
 
         if not elements:
             elements = app.project.get_default_targets()
@@ -649,7 +663,7 @@ def show(app, elements, deps, except_, order, format_):
 
         # Don't spend time interrogating the cache if we don't need to show element state
         if need_state:
-            app.stream.query_cache(dependencies)
+            app.stream.query_cache(dependencies, need_state=True)
 
         if order == "alpha":
             dependencies = sorted(dependencies)
