@@ -56,7 +56,7 @@ _PROJECT_CONF_FILE = "project.conf"
 class ProjectConfig:
     def __init__(self):
         self.options = None  # OptionPool
-        self.base_variables = {}  # The base set of variables
+        self.base_variables = None  # The base set of variables
         self.element_overrides = {}  # Element specific configurations
         self.source_overrides = {}  # Source specific configurations
         self.mirrors = {}  # Dictionary of SourceMirror objects
@@ -545,6 +545,15 @@ class Project:
             return
         assert self._partially_loaded
 
+        # Ensure that the parent project is fully loaded first
+        if self.junction:
+            self.junction._get_project().ensure_fully_loaded()
+
+            if self._fully_loaded:
+                # The parent project triggered the loading of this project,
+                # don't load this subproject twice.
+                return
+
         # Here we mark the project as fully loaded right away,
         # before doing the work.
         #
@@ -560,9 +569,6 @@ class Project:
         #
         self._fully_loaded = True
 
-        if self.junction:
-            self.junction._get_project().ensure_fully_loaded()
-
         self._load_second_pass()
 
     # get_default_target()
@@ -1069,16 +1075,7 @@ class Project:
 
         # Extend variables with automatic variables and option exports
         # Initialize it as a string as all variables are processed as strings.
-        # Based on some testing (mainly on AWS), maximum effective
-        # max-jobs value seems to be around 8-10 if we have enough cores
-        # users should set values based on workload and build infrastructure
-        if self._context.build_max_jobs == 0:
-            # User requested automatic max-jobs
-            platform = self._context.platform
-            output.base_variables["max-jobs"] = str(platform.get_cpu_count(8))
-        else:
-            # User requested explicit max-jobs setting
-            output.base_variables["max-jobs"] = str(self._context.build_max_jobs)
+        output.base_variables["max-jobs"] = str(self._context.effective_build_max_jobs)
 
         # Export options into variables, if that was requested
         output.options.export_variables(output.base_variables)
