@@ -349,14 +349,6 @@ class Cli:
             if env is not None:
                 stack.enter_context(environment(env))
 
-            # Ensure we have a working stdout - required to work
-            # around a bug that appears to cause AIX to close
-            # sys.__stdout__ after setup.py
-            try:
-                sys.__stdout__.fileno()
-            except ValueError:
-                sys.__stdout__ = open("/dev/stdout", "w", encoding="utf-8")  # pylint: disable=consider-using-with
-
             result = self._invoke(bst_cli, bst_args, binary_capture=binary_capture)
 
         # Some informative stdout we can observe when anything fails
@@ -640,7 +632,14 @@ class TestArtifact:
 
         normal_name = element_name.replace(os.sep, "-")
         cache_dir = os.path.splitext(os.path.join(cache_dir, "test", normal_name))[0]
-        shutil.rmtree(cache_dir)
+
+        # Don't cause a failure here if the artifact is not there, we use this in integration
+        # tests where the cache is shared and we just want to clean up some artifacts
+        # regardless of whether they have already been built or not.
+        try:
+            shutil.rmtree(cache_dir)
+        except FileNotFoundError:
+            pass
 
     # is_cached():
     #
