@@ -85,13 +85,18 @@ class Sandbox:
         self.__env = None  # type: Optional[Dict[str, str]]
         self.__mount_sources = {}  # type: Dict[str, str]
         self.__allow_run = True
+        self.__subsandboxes = []  # type: List[Sandbox]
 
         # Plugin element full name for logging
         plugin = kwargs.get("plugin", None)
         if plugin:
             self.__element_name = plugin._get_full_name()
         else:
-            self.__element_name = None
+            parent = kwargs.get("parent", None)
+            if parent:
+                self.__element_name = parent._get_element_name()
+            else:
+                self.__element_name = None
 
         # Configuration from kwargs common to all subclasses
         self.__config = kwargs["config"]
@@ -115,7 +120,7 @@ class Sandbox:
     # Called when exiting the with-statement context.
     #
     def __exit__(self, exc_type, exc_value, traceback) -> None:
-        self._cleanup()
+        pass
 
     def get_virtual_directory(self) -> Directory:
         """Fetches the sandbox root directory as a virtual Directory.
@@ -171,7 +176,7 @@ class Sandbox:
         root_read_only: bool = False,
         cwd: Optional[str] = None,
         env: Optional[Dict[str, str]] = None,
-        label: str = None
+        label: Optional[str] = None
     ) -> Optional[int]:
         """Run a command in the sandbox.
 
@@ -214,7 +219,7 @@ class Sandbox:
 
     @contextmanager
     def batch(
-        self, *, root_read_only: bool = False, label: str = None, collect: str = None
+        self, *, root_read_only: bool = False, label: Optional[str] = None, collect: Optional[str] = None
     ) -> Generator[None, None, None]:
         """Context manager for command batching
 
@@ -268,13 +273,6 @@ class Sandbox:
     #    Abstract Methods for Sandbox implementations   #
     #####################################################
 
-    # _cleanup():
-    #
-    # Abstract method to release resources when the sandbox is discarded
-    #
-    def _cleanup(self):
-        pass
-
     # _run()
     #
     # Abstract method for running a single command
@@ -345,7 +343,7 @@ class Sandbox:
         flags: int,
         cwd: Optional[str] = None,
         env: Optional[Dict[str, str]] = None,
-        label: str = None
+        label: Optional[str] = None
     ) -> Optional[int]:
         if not self.__allow_run:
             raise _SandboxBug("Element specified BST_RUN_COMMANDS as False but called Sandbox.run()")
@@ -445,6 +443,16 @@ class Sandbox:
 
         return env
 
+    # _get_configured_environment()
+    #
+    # Return the environment exactly as configured with `set_environment()`,
+    # or `None` if `set_environment()` has not been called.
+    #
+    # Returns
+    #   (Dict[str, str]): The configured environment
+    def _get_configured_environment(self):
+        return self.__env
+
     # _get_work_directory()
     #
     # Fetches the working directory for running commands
@@ -565,6 +573,28 @@ class Sandbox:
     def _disable_run(self):
         self.__allow_run = False
 
+    # _create_subsandbox()
+    #
+    # Create an empty sandbox
+    #
+    # This allows an element to use a secondary sandbox for manipulating artifacts
+    # that does not affect the build sandbox.
+    #
+    def _create_subsandbox(self, **kwargs):
+        sub = Sandbox(
+            self.__context,
+            self.__project,
+            parent=self,
+            stdout=self.__stdout,
+            stderr=self.__stderr,
+            config=self.__config,
+        )
+        self.__subsandboxes.append(sub)
+        return sub
+
+    def _get_subsandboxes(self):
+        return self.__subsandboxes
+
 
 # SandboxFlags()
 #
