@@ -13,12 +13,6 @@
 #
 #  Authors:
 #        Tristan Van Berkom <tristan.vanberkom@codethink.co.uk>
-
-# Disable this for the file, because pylint is not picking it up
-# when specifying it on the specific line.
-#
-# pylint: disable=subprocess-popen-preexec-fn
-#
 """
 Utilities
 =========
@@ -41,7 +35,7 @@ import threading
 import itertools
 from contextlib import contextmanager
 from pathlib import Path
-from typing import Callable, IO, Iterable, Iterator, Optional, Tuple, Union
+from typing import Callable, IO, Iterable, Iterator, Optional, Tuple, Union, Pattern
 from google.protobuf import timestamp_pb2
 
 import psutil
@@ -72,6 +66,10 @@ os.umask(_UMASK)
 # it might not work
 _USE_CP_FILE_RANGE = hasattr(os, "copy_file_range")
 
+# The default version guessing pattern for utils.guess_version()
+#
+_DEFAULT_GUESS_PATTERN = re.compile(r"(\d+)\.(\d+)(?:\.(\d+))?")
+
 
 class UtilError(BstError):
     """Raised by utility functions when system calls fail.
@@ -228,7 +226,7 @@ def list_relative_paths(directory: str) -> Iterator[str]:
 
 # pylint: disable=anomalous-backslash-in-string
 def glob(paths: Iterable[str], pattern: str) -> Iterator[str]:
-    """A generator to yield paths which match the glob pattern
+    r"""A generator to yield paths which match the glob pattern
 
     Args:
        paths (iterable): The paths to check
@@ -703,15 +701,87 @@ def save_file_atomic(
 
 # get_umask():
 #
-# Get the process's file mode creation mask without changing it.
+#
 #
 # Returns:
-#     (int) The process's file mode creation mask.
+#     (int)
 #
-def get_umask():
+def get_umask() -> int:
+    """
+    Get the process's file mode creation mask without changing it.
+
+    Returns: The process's file mode creation mask.
+    """
     return _UMASK
 
 
+def guess_version(string: str, *, pattern: Optional[Pattern[str]] = None) -> Optional[str]:
+    """
+    Attempt to extract a version from an arbitrary string.
+
+    This function is used by sources who implement
+    :func:`Source.get_source_info() <buildstream.source.SourceFetcher.get_source_info>`
+    in order to provide a guess at what the version is, given some domain specific
+    knowledge such as a git tag or a tarball URL.
+
+    This function will be traverse the provided string for non-overlapping matches, and
+    in the case of *optional groups* being specified in the pattern; the match with the
+    greatest amount of matched groups will be preferred, allowing for correct handling
+    of cases like: ``https://example.com/releases/1.2/release-1.2.3.tgz`` which may
+    match the *pattern* multiple times.
+
+    The resulting version will be the captured groups, separated by ``.`` characters.
+
+    Args:
+       string: The domain specific string to scan for a version
+       pattern: A compiled regex pattern to scan *string*, or None for the default ``(\\d+)\\.(\\d+)(?:\\.(\\d+))?``.
+
+    Returns:
+       The guessed version, or None if no match was found.
+
+    .. note::
+
+       **Specifying a pattern**
+
+       When specifying the pattern, any number of capture groups may be specified, and
+       the match containing the most matching groups will be selected.
+
+       The capture groups must contain only the intended result and not any separating
+       characters.
+
+       For example, you may parse a string such as ``release-1_2_3-r2`` with the pattern:
+       ``(\\d+)_(\\d+)(?:_(\\d+))?(?:\\-(r\\d+))?``, and this would produce the parsed
+       version ``1.2.3.r2``.
+
+    **Since: 2.5**.
+    """
+    version_guess: Optional[str] = None
+    version_guess_groups = 0
+
+    if pattern is None:
+        pattern = _DEFAULT_GUESS_PATTERN
+
+    # Iterate over non-overlapping matches, and prefer a match which is more qualified (i.e. 1.2.3 is better than 1.2)
+    for version_match in pattern.finditer(string):
+
+        if not version_match:
+            iter_guess = None
+            iter_n_groups = 0
+        elif pattern.groups == 0:
+            iter_guess = str(version_match.group(0))
+            iter_n_groups = 1
+        else:
+            iter_groups = [group for group in version_match.groups() if group is not None]
+            iter_n_groups = len(iter_groups)
+            iter_guess = ".".join(iter_groups)
+
+        if version_guess is None or iter_n_groups > version_guess_groups:
+            version_guess = iter_guess
+            version_guess_groups = iter_n_groups
+
+    return version_guess
+
+
 # _get_host_tool_internal():
 #
 # Get the full path of a host tool, including tools bundled inside the Python package.
@@ -832,6 +902,35 @@ def _parse_size(size, volume):
     return int(num) * 1024 ** units.index(unit)
 
 
+# _parse_percentage():
+#
+# Convert a string representing a percentage between 0% and 100% to a float.
+# E.g. "80%" -> 0.8.
+#
+# Arguments:
+#     percentage (str) The string to parse
+#
+# Returns:
+#     (float) The percentage as a float
+#
+# Raises:
+#     UtilError if the string is not a valid percentage.
+#
+def _parse_percentage(percentage):
+    if not percentage.endswith("%"):
+        raise UtilError("{} is not a valid percentage.".format(percentage))
+
+    try:
+        num = float(percentage[:-1])
+    except ValueError:
+        raise UtilError("{} is not a valid percentage.".format(percentage))
+
+    if num < 0 or num > 100:
+        raise UtilError("{} is not between 0% and 100%.".format(percentage))
+
+    return num / 100
+
+
 # _pretty_size()
 #
 # Converts a number of bytes into a string representation in KiB, MiB, GiB, TiB
@@ -1101,37 +1200,6 @@ def _set_deterministic_user(directory):
             os.chown(os.path.join(root, dirname), user, group, follow_symlinks=False)
 
 
-# _set_deterministic_mtime()
-#
-# Set the mtime for every file in a directory tree to the same.
-#
-# Args:
-#    directory (str): The directory to recursively set the mtime on
-#
-def _set_deterministic_mtime(directory):
-    for dirname, _, filenames in os.walk(directory.encode("utf-8"), topdown=False):
-        for filename in filenames:
-            pathname = os.path.join(dirname, filename)
-
-            # Python's os.utime only ever modifies the timestamp
-            # of the target, it is not acceptable to set the timestamp
-            # of the target here, if we are staging the link target we
-            # will also set its timestamp.
-            #
-            # We should however find a way to modify the actual link's
-            # timestamp, this outdated python bug report claims that
-            # it is impossible:
-            #
-            #   http://bugs.python.org/issue623782
-            #
-            # However, nowadays it is possible at least on gnuish systems
-            # with with the lutimes glibc function.
-            if not os.path.islink(pathname):
-                os.utime(pathname, (BST_ARBITRARY_TIMESTAMP, BST_ARBITRARY_TIMESTAMP))
-
-        os.utime(dirname, (BST_ARBITRARY_TIMESTAMP, BST_ARBITRARY_TIMESTAMP))
-
-
 # _tempdir()
 #
 # A context manager for doing work in a temporary directory.
@@ -1213,66 +1281,6 @@ def _tempnamedfile(mode="w+b", encoding=None, suffix="", prefix="tmp", dir=None)
         yield temp
 
 
-# _tempnamedfile_name()
-#
-# A context manager for doing work on a temporary file, via the filename only.
-#
-# Note that a Windows restriction prevents us from using both the file
-# descriptor and the file name of tempfile.NamedTemporaryFile, the same file
-# cannot be opened twice at the same time. This wrapper makes it easier to
-# operate in a Windows-friendly way when only filenames are needed.
-#
-# Takes care to delete the file even if interrupted by SIGTERM. Note that there
-# is a race-condition, so this is done on a best-effort basis.
-#
-# Args:
-#    dir (str): A path to a parent directory for the temporary file, this is
-#               not optional for security reasons, please see below.
-#
-# Note that 'dir' should not be a directory that may be shared with potential
-# attackers that have rights to affect the directory, e.g. the system temp
-# directory. This is because an attacker can replace the temporary file with
-# e.g. a symlink to another location, that the BuildStream user has rights to
-# but the attacker does not.
-#
-# See here for more information:
-# https://www.owasp.org/index.php/Insecure_Temporary_File
-#
-# Yields:
-#    (str): The temporary file name
-#
-@contextmanager
-def _tempnamedfile_name(dir):  # pylint: disable=redefined-builtin
-    filename = None
-
-    def rm_tempfile():
-        nonlocal filename
-        if filename is not None:
-            # Note that this code is re-entrant - it's possible for us to
-            # "delete while we're deleting" if we are responding to SIGTERM.
-            #
-            # For simplicity, choose to leak the file in this unlikely case,
-            # rather than introducing a scheme for handling the file sometimes
-            # being missing. Note that we will still leak tempfiles on forced
-            # termination anyway.
-            #
-            # Note that we must not try to delete the file more than once. If
-            # we delete the file then another temporary file can be created
-            # with the same name.
-            #
-            filename2 = filename
-            filename = None
-            os.remove(filename2)
-
-    with _signals.terminator(rm_tempfile):
-        fd, filename = tempfile.mkstemp(dir=dir)
-        os.close(fd)
-        try:
-            yield filename
-        finally:
-            rm_tempfile()
-
-
 # _kill_process_tree()
 #
 # Brutally murder a process and all of its children
@@ -1322,14 +1330,7 @@ def _call(*popenargs, terminate=False, **kwargs):
 
     process = None
 
-    old_preexec_fn = kwargs.get("preexec_fn")
-    if "preexec_fn" in kwargs:
-        del kwargs["preexec_fn"]
-
-    def preexec_fn():
-        os.umask(stat.S_IWGRP | stat.S_IWOTH)
-        if old_preexec_fn is not None:
-            old_preexec_fn()
+    kwargs.setdefault("umask", stat.S_IWGRP | stat.S_IWOTH)
 
     # Handle termination, suspend and resume
     def kill_proc():
@@ -1375,7 +1376,7 @@ def _call(*popenargs, terminate=False, **kwargs):
             os.killpg(group_id, signal.SIGCONT)
 
     with _signals.suspendable(suspend_proc, resume_proc), _signals.terminator(kill_proc), subprocess.Popen(
-        *popenargs, preexec_fn=preexec_fn, universal_newlines=True, **kwargs
+        *popenargs, universal_newlines=True, **kwargs
     ) as process:
         # Here, we don't use `process.communicate()` directly without a timeout
         # This is because, if we were to do that, and the process would never
@@ -1492,14 +1493,6 @@ def _deduplicate(iterable, key=None):
                 yield element
 
 
-# Like os.path.getmtime(), but returns the mtime of a link rather than
-# the target, if the filesystem supports that.
-#
-def _get_link_mtime(path):
-    path_stat = os.lstat(path)
-    return path_stat.st_mtime
-
-
 # _message_digest()
 #
 # Args:
